@@ -31,7 +31,7 @@ class BinaryTreePrinter:
         self.right_wing_tail = right_wing_tail
         self.tree_depth = 0
         self.max_data_string_length = 0
-        self.char_map = None
+        self.char_map = list()
         self.node_to_string = node_to_string if node_to_string is not None else lambda node: str(node.__dict__[self.data_name])
 
     def get_left(self, node):
@@ -91,9 +91,7 @@ class BinaryTreePrinter:
         return max(left_depth, right_depth) + 1
 
     def _make_char_map(self, node):
-        self.char_map = list()
-        if node is None:
-            return
+        self.char_map, self.max_data_string_length = list(), 0
         self.tree_depth = self._collect_tree_info(node)
         last_line_data_count = pow(2, self.tree_depth - 1)
         last_line_string_length = last_line_data_count * (self.max_data_string_length + 1) - 1  # +one space in between
@@ -122,18 +120,20 @@ class BinaryTreePrinter:
                 depth += 1
 
     def to_string(self, node, trim_left=True, trim_right=True):
+        if node is None:
+            return "None\n"
         self._make_char_map(node)
         start_print_index, end_print_index = 0, len(self.char_map[0])
         if trim_left:  # trim left blank
             for line in self.char_map:
-                last_non_blank_index_from_begin = 0
+                last_non_blank_index_from_start = 0
                 for char in line:
                     if char == " ":
-                        last_non_blank_index_from_begin += 1
+                        last_non_blank_index_from_start += 1
                     else:
                         break
-                if start_print_index == 0 or last_non_blank_index_from_begin < start_print_index:
-                    start_print_index = last_non_blank_index_from_begin
+                if start_print_index == 0 or last_non_blank_index_from_start < start_print_index:
+                    start_print_index = last_non_blank_index_from_start
         if trim_right:  # trim right blank
             for line in self.char_map:
                 last_non_blank_index_from_end = len(self.char_map[0])
